@@ -1,3 +1,24 @@
+/* boucequeue.c - This program does the non-delivery deports spooling
+  RCS Identication ; $Revision: 1.3 $ ; $Date: 2001/12/04 13:19:03 $ 
+
+  Sympa - SYsteme de Multi-Postage Automatique
+  Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
+  Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+
 #include <stdio.h>
 #include <ctype.h>
 #include <fcntl.h>
