@@ -1,24 +1,31 @@
-#!--PERL--
+# List.pm -  This module provides parsing functions for sympa template files.
+# These templates consists of text files including variables and directives
+# (IF, FOREACH, INCLUDE, PARSE,...)
+# RCS Identication ; $Revision: 1.5 $ ; $Date: 2002/01/17 12:53:05 $ 
+#
+# Sympa - SYsteme de Multi-Postage Automatique
+# Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
+# Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 use FileHandle;
 
-## 21/10/99  INCLUDE et PARSE manipule des noms de fichiers et non des 
-## variables
-## 04/10/99  INCLUDE is now PARSE, INCLUDE preserves content of file
-## Version Mai 1999
-## 15/11/99  [IF ! var]
-##           spaces accepted whithin []                  
-## 16/11/99  Don't process empty FOREACH
-## 17/11/99  Added [ELSIF...]
-## 22/11/99  force Single-line regexp search
-
 my ($index, @t, $data, $internal);
 
-## Routine Perl permettant d'interpreter un modele de document HTML ou autre
-## pouvant contenir des variables ainsi que des directives IF, FOREACH, INCLUDE, PARSE
-## Copyright Comite Reseau des Universites 1999
-## Olivier.Salaun@cru.fr
-
+## The main parsing sub
 sub parse_tpl {
     my ($template, $output);
     ($data, $template, $output) = @_;
@@ -63,6 +70,22 @@ sub parse_tpl {
 
 return 1;
 
+## Processes [SET xx=yy] 
+sub do_setvar {
+    my $echo = shift;
+
+    if ($echo && /\[\s*SET\s+(\w+)\s*\=\s*(\w+)\->(\w+)\s*\]/i) {
+	if (ref ($data->{$2})) {
+	    $data->{$1} = $data->{$2}{$3};
+	}
+    }elsif ($echo && /\[\s*SET\s+(\w+)\s*\=\s*(\d+)\s*\]/i) {
+	$data->{$1} = $2;
+    }
+
+    return;
+}
+
+## Processe [INCLUDE 'file']
 sub do_include {
     my $file = pop;
 
@@ -72,6 +95,7 @@ sub do_include {
     close $fh;
 }
 
+## Processe [IF ...]
 ## $echo possible values : 0, 1, -1
 sub do_if {
     my ($echo) = @_;
@@ -96,16 +120,21 @@ sub do_if {
 		    $echo *= -1 if (! $data->{$2});
 		    
 		}elsif (/\[\s*(IF|ELSIF)\s+(\w+)\-\>(\w+)\s*\]/i) {
-		    $echo *= -1 if (defined $data->{$2}{$3});
+		    $echo *= -1 if (defined $data->{$2} && $data->{$2}{$3});
 		    
 		}elsif (/\[\s*(IF|ELSIF)\s+!\s*(\w+)\-\>(\w+)\s*\]/i) {
-		    $echo *= -1 if (! defined $data->{$2}{$3});
+		    $echo *= -1 if ((! defined $data->{$2}) || (! $data->{$2}{$3}));
 		    
 		}elsif (/\[\s*(IF|ELSIF)\s+(\w+)\s*(=|<>)\s*(\S+)\s*\]/i) {
 		    $echo *= -1  if ( ( ($3 eq "=") and ($data->{$2} eq $4) ) 
 				     or  
 				     ( ($3 eq "<>") and ($data->{$2} ne $4) ) );
 		    
+		}elsif (/\[\s*(IF|ELSIF)\s+(\w+)\-\>(NAME|INDEX)\s*(=|<>)\s*(\S+)\s*\]/i) {
+		    $echo *= -1 if ( ( ($4 eq "=") and ($internal->{$2}{$3} eq $5) ) 
+				     or  
+				     ( ($4 eq "<>") and ($internal->{$2}{$3} ne $5) ) );
+	    
 		}elsif (/\[\s*(IF|ELSIF)\s+(\w+)\-\>(\w+)\s*(=|<>)\s*(\S+)\s*\]/i) {
 		    $echo *= -1 if ( ( ($4 eq "=") and ($data->{$2}{$3} eq $5) ) 
 				     or  
@@ -125,9 +154,10 @@ sub do_if {
     return;
 }
 
+## Processe [FOREACH x IN y]
 sub do_foreach {
     my ($echo) = @_;
-    my ($i, $val, $var, $hash, $start);
+    my ($i, $val, $var, $struct, $start);
 
     print STDERR "[$index]\tsub do_foreach ($echo)\n" if $opt_p;
     print STDERR "\t$t[$index]" if $opt_p;
@@ -145,6 +175,7 @@ sub do_foreach {
 		$struct = $data->{$key};
 	    }
 
+	    my ($prev_data, $prev_internal) = ($data->{$var}, $internal->{$var});
 	    foreach $i (sort keys %{$struct}) {
 		$data->{$var} = $struct->{$i};
 
@@ -160,6 +191,9 @@ sub do_foreach {
 		    &process($echo);
 		}
 	    }
+	    ## Restore context
+	    ($data->{$var}, $internal->{$var}) = ($prev_data, $prev_internal);
+
 	}elsif (($key2 and (ref($data->{$key}{$key2}) =~ /ARRAY/))
 		or (! $key2 and ref($data->{$key}) =~ /ARRAY/)) {
 
@@ -171,6 +205,7 @@ sub do_foreach {
 		$struct = $data->{$key};
 	    }
 
+	    my ($prev_data, $prev_internal) = ($data->{$var}, $internal->{$var});
 	    foreach $val (@{$struct}) {
 		$data->{$var} = $val;
 		$internal->{$var}{'INDEX'} = $i;
@@ -187,7 +222,11 @@ sub do_foreach {
 
 		$i++;
 	    }
+	    ## Restore context
+	    ($data->{$var}, $internal->{$var}) = ($prev_data, $prev_internal);
+
 	}else {
+
 	    while ($_ = $t[$index]) {
 		if (/\[END\]/i) {
 		    last;
@@ -195,15 +234,32 @@ sub do_foreach {
 		
 		&process(0);
 	    }
-
 	}
     }
     return;
 }
 
+## Processes [STOPPARSE]
+sub do_stopparse {
+    print STDERR "[$index]\tsub do_stopparse\n" if $opt_p;
+    print STDERR "\t$t[$index]" if $opt_p;
+
+    $index++;
+
+    while ($_ = $t[$index]) {
+	return if /\[\s*STARTPARSE\s*\]/i;
+	print;
+	$index++;
+    }
+    return;
+}
+
+## Main processing sub
 sub process {
     my ($echo) = @_;
 
+    my $backup_echo;
+
     $echo = 0 if ($echo == -1);
 
     print STDERR "[$index]\tsub process ($echo)\n" if $opt_p;
@@ -223,12 +279,18 @@ sub process {
 	    &do_include($1) if ($echo == 1);
 	}elsif (/\[\s*PARSE\s+(\w+)\s*\]/i) {
 	    parse_tpl($data, $data->{$1}, select()) if ($echo == 1);
+	}elsif (/\[\s*PARSE\s+(\w+)\->(\w+)\s*\]/i) {
+	    parse_tpl($data, $data->{$1}{$2}, select()) if (defined $data->{$1} && $echo == 1);
 	}elsif (/\[\s*PARSE\s+\'(\S+)\'\s*\]/i) {
 	    parse_tpl($data, $1, select())  if ($echo == 1);
 	}elsif (/\[\s*FOREACH\s+(\w+)\s+IN\s+(\w+(\->\w+)?)\s*\]/i) {
 	    &do_foreach($echo);
 	}elsif (/\[\s*END\s*\]/i) {
 	    return;
+	}elsif (/\[\s*STOPPARSE\s*\]/i) {
+	    &do_stopparse();
+	}elsif (/\[\s*SET\s+(\w+)\s*\=\s*(\w+\->\w+|\d+)\s*\]/i) {
+	    &do_setvar($echo);
 	}elsif ($echo == 1) {
 	    &do_parse();
 	    print;
@@ -237,42 +299,35 @@ sub process {
     return;
 }
 
-
+## Instanciates variables
 sub do_parse {
 
     print STDERR "[$index]\tsub do_parse\n" if $opt_p;
     print STDERR "\t$t[$index]" if $opt_p;
 
     $_ = $t[$index];
-
     
-    while (/\[\s*(\w+)\-\>(INDEX|NAME)\s*\]/g) {
-	my ($v1, $v2) = ($1, $2);
-	
-        if (ref($internal->{$v1}) eq 'HASH') {
-	    s/\[($v1)\-\>($v2)\]/$internal->{$v1}{$v2}/;
-	}else {
-	    s/\[($v1)\-\>($v2)\]//;
-	}
-    }
-
-    while (/\[\s*(\w+)\-\>(\w+)\s*\]/g) {
-	my ($v1, $v2) = ($1, $2);
-	
-        if (ref($data->{$v1}) eq 'HASH') {
-	    s/\[($v1)\-\>($v2)\]/$data->{$v1}{$v2}/;
-	}else {
-	    s/\[($v1)\-\>($v2)\]//;
-	}
-    }
-
-    while (/\[\s*(\w+)\s*\]/g) {
-	my $v = $1;
-	
-	s/\[$v\]/$data->{$v}/;
-    }
-
-    return;
+    s/\[(\w+\-\>\w+|\w+)\]/&do_eval($1)/eg;
+    
+    return $_;
 }
 
+sub do_eval {
+    my $var = shift;
 
+    if ($var =~ /^(\w+)\-\>(INDEX|NAME)$/) {
+	if (ref($internal->{$1}) eq 'HASH') {
+	    return $internal->{$1}{$2};
+	}
+    }elsif ($var =~ /^(\w+)\-\>(\w+)$/) {
+	if (ref($data->{$1}) eq 'HASH') {
+	    return $data->{$1}{$2};
+	}
+    }elsif ($var =~ /^(\w+)$/) {
+	return $data->{$1};
+    }else {
+	print STDERR "Unable to parse '$var'\n" if $opt_p;
+    }
+
+    return '';
+}
