@@ -1,3 +1,24 @@
+/* queue.c - This program does the messages spooling
+   RCS Identication ; $Revision: 1.3 $ ; $Date: 2001/12/04 13:19:03 $ 
+
+   Sympa - SYsteme de Multi-Postage Automatique
+   Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
+   Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+
 #include <stdio.h>
 #include <ctype.h>
 #include <fcntl.h>
@@ -5,7 +26,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-static char rcsid[] = "(@)$Id: queue.c,v 1.1.1.1 1998/09/18 09:16:06 sympa Exp $";
+static char rcsid[] = "(@)$Id: queue.c,v 1.3 2001/12/04 13:19:03 aumont Exp $";
 
 static char     qfile[128];
 static char     buf[16384];
@@ -65,6 +86,8 @@ main(int argn, char **argv)
 
    /* Usage : queue list-name */
    if ((argn < 2) || (argn >3)) {
+     fprintf(stderr,"%s: usage error, one one list-name argument expected.\n",
+            argv[0]);
       exit(EX_TEMPFAIL);
    }
 
@@ -81,16 +104,26 @@ main(int argn, char **argv)
          strcpy(listname, argv[2]);
    }
 
-   if ((queuedir = readconf(CONFIG)) == NULL)
+   if ((queuedir = readconf(CONFIG)) == NULL){
+     fprintf(stderr,"%s: cannot read configuration file '%s'.\n",
+            argv[0],CONFIG);
       exit(EX_TEMPFAIL);
+   }
    if (chdir(queuedir) == -1) {
-      exit(EX_TEMPFAIL);
+     char* buffer=(char*)malloc(strlen(argv[0])+strlen(queuedir)+80);
+     sprintf(buffer,"%s: while changing dir to '%s'",argv[0],queuedir);
+     perror(buffer);
+     exit(EX_TEMPFAIL);
    }
    umask(027);
    sprintf(qfile, "T.%s.%ld.%d", listname, time(NULL), getpid());
    fd = open(qfile, O_CREAT|O_WRONLY, 0600);
-   if (fd == -1)
-      exit(EX_TEMPFAIL);
+   if (fd == -1){
+     char* buffer=(char*)malloc(strlen(argv[0])+strlen(queuedir)+80);
+     sprintf(buffer,"%s: while opening queue file '%s'",argv[0],qfile);
+     perror(buffer);
+     exit(EX_TEMPFAIL);
+   }
    write(fd, "X-Sympa-To: ", 12);
    write(fd, listname, strlen(listname));
    write(fd, "\n", 1);
